#!usr/bin/python
# -*- coding: utf-8 -*-
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTIBILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.

"Almacenamiento de duplicados electrónicos RG1361/02 y RG1579/03 AFIP"

__author__ = "Mariano Reingart (reingart@gmail.com)"
__copyright__ = "Copyright (C) 2009-2015 Mariano Reingart"
__license__ = "GPL 3.0"
__version__ = "1.22d"

LICENCIA = """
sired.py: Generador de archivos ventas para SIRED/SIAP RG1361/02 RG1579/03
(Sistema Resúmen Electrónico de Datos / Almacenamiento de Duplicados)
Copyright (C) 2009-2015 Mariano Reingart reingart@gmail.com

Este progarma es software libre, se entrega ABSOLUTAMENTE SIN GARANTIA
y es bienvenido a redistribuirlo bajo la licencia GPLv3.

Para información adicional sobre garantía, soporte técnico comercial
e incorporación/distribución en programas propietarios ver PyAfipWs:
http://www.sistemasagiles.com.ar/trac/wiki/PyAfipWs
"""

import csv
import datetime
from decimal import Decimal
import os
import sys
import unicodedata
import sqlite3
import traceback

from .utils import leer, escribir, C, N, A, I, B, get_install_dir

CUIT = '20267565393'

# ESPECIFICACIONES TECNICAS - ANEXO II RESOLUCION GENERAL N°1361
# http://www.afip.gov.ar/afip/resol136102_Anexo_II.html

categorias = {"responsable inscripto": "01", # IVA Responsable Inscripto
              "responsable no inscripto": "02", # IVA Responsable no Inscripto
              "no responsable": "03", # IVA no Responsable
              "exento": "04", # IVA Sujeto Exento
              "consumidor final": "05", # Consumidor Final
              "monotributo": "06", # Responsable Monotributo
              "responsable monotributo": "06", # Responsable Monotributo
              "no categorizado": "07", # Sujeto no Categorizado
              "importador": "08", # Importador del Exterior
              "exterior": "09", # Cliente del Exterior
              "liberado": "10", # IVA Liberado Ley Nº 19.640
              "responsable inscripto - agente de percepción": "11", # IVA Responsable Inscripto - Agente de Percepcion
}

codigos_operacion = {
    "Z": "Exportaciones a la zona franca",
    "X": "Exportaciones al Exterior",
    "E": "Operaciones Exentas",
}

CAB_FAC_TIPO1 = [
    ('tipo_reg', 1, N),
    ('fecha_cbte', 8, N),
    ('tipo_cbte', 2, N),
    ('ctl_fiscal', 1, C),
    ('punto_vta', 4, N),
    ('cbt_numero', 8, N),
    ('cbte_nro_reg', 8, N),
    ('cant_hojas', 3, N),
    ('tipo_doc', 2, N),
    ('nro_doc', 11, N),
    ('nombre', 30, A),
    ('imp_total', 15, I),
    ('imp_tot_conc', 15, I),
    ('imp_neto', 15, I),
    ('impto_liq', 15, I),
    ('impto_liq_rni', 15, I),
    ('imp_op_ex', 15, I),
    ('impto_perc', 15, I),
    ('imp_iibb', 15, I),
    ('impto_perc_mun', 15, I),
    ('imp_internos', 15, I),
    ('transporte', 15, I),
    ('categoria', 2, N),
    ('imp_moneda_id', 3, A),
    ('imp_moneda_ctz', 10, I),
    ('alicuotas_iva', 1, N),
    ('codigo_operacion', 1, C),
    ('cae', 14, N),
    ('fecha_vto', 8, N),
    ('fecha_anulacion', 8, A),
    ]

# campos especiales del encabezado:
IMPORTES = ('imp_total', 'imp_tot_conc', 'imp_neto', 'impto_liq',
            'impto_liq_rni', 'imp_op_ex', 'impto_perc', 'imp_iibb',
            'impto_perc_mun', 'imp_internos')

# total
CAB_FAC_TIPO2 = [
    ('tipo_reg', 1, N),
    ('periodo', 6, N),
    ('relleno', 13, B),
    ('cant_reg_tipo_1', 8, N),
    ('relleno', 17, B),
    ('cuit', 11, N),
    ('relleno', 22, B),
    ('imp_total', 15, I),
    ('imp_tot_conc', 15, I),
    ('imp_neto', 15, I),
    ('impto_liq', 15, I),
    ('impto_liq_rni', 15, I),
    ('imp_op_ex', 15, I),
    ('impto_perc', 15, I),
    ('imp_iibb', 15, I),
    ('impto_perc_mun', 15, I),
    ('imp_internos', 15, I),
    ('relleno', 62, B),
    ]

DETALLE = [
    ('tipo_cbte', 2, N),
    ('ctl_fiscal', 1, C),
    ('fecha_cbte', 8, N),
    ('punto_vta', 4, N),
    ('cbt_numero', 8, N),
    ('cbte_nro_reg', 8, N),
    ('qty', 12, I),
    ('pro_umed', 2, N),
    ('pro_precio_uni', 16, I),
    ('imp_bonif', 15, I),
    ('imp_ajuste', 16, I),
    ('imp_total', 16, I),
    ('alicuota_iva', 4, I),
    ('gravado', 1, C),
    ('anulacion', 1, C),
    ('codigo', 50, A),
    ('ds', 150, A),
    ]

VENTAS_TIPO1 = [
    ('tipo_reg', 1, N),
    ('fecha_cbte', 8, N),
    ('tipo_cbte', 2, N),
    ('ctl_fiscal', 1, C),
    ('punto_vta', 4, N),
    ('cbt_numero', 20, N),
    ('cbte_nro_reg', 20, N),
    ('tipo_doc', 2, N),
    ('nro_doc', 11, N),
    ('nombre', 30, A),
    ('imp_total', 15, I),
    ('imp_tot_conc', 15, I),
    ('imp_neto', 15, I),
    ('alicuota_iva', 4, I),
    ('impto_liq', 15, I),
    ('impto_liq_rni', 15, I),
    ('imp_op_ex', 15, I),
    ('impto_perc', 15, I),
    ('imp_iibb', 15, I),
    ('impto_perc_mun', 15, I),
    ('imp_internos', 15, I),
    ('categoria', 2, N),
    ('imp_moneda_id', 3, A),
    ('imp_moneda_ctz', 10, I),
    ('alicuotas_iva', 1, N),
    ('codigo_operacion', 1, C),
    ('cae', 14, N),
    ('fecha_vto', 8, N),
    ('fecha_anulacion', 8, A),
    ('info_adic', 75-0, B),
    ]

VENTAS_TIPO2 = [
    ('tipo_reg', 1, N),
    ('periodo', 6, N),
    ('relleno', 29, B),
    ('cant_reg_tipo_1', 12, N),
    ('relleno', 10, B),
    ('cuit', 11, N),
    ('relleno', 30, B),
    ('imp_total', 15, I),
    ('imp_tot_conc', 15, I),
    ('imp_neto', 15, I),
    ('Relleno', 4, B),
    ('impto_liq', 15, I),
    ('impto_liq_rni', 15, I),
    ('imp_op_ex', 15, I),
    ('impto_perc', 15, I),
    ('imp_iibb', 15, I),
    ('impto_perc_mun', 15, I),
    ('imp_internos', 15, I),
    ('relleno', 122, B),
    ]

# Regimen de informacion de compras y ventas

REGINFO_CV_VENTAS_CBTE = [
    ('fecha_cbte', 8, N),
    ('tipo_cbte', 3, N),
    ('punto_vta', 5, N),
    ('cbt_numero_desde', 20, N),
    ('cbt_numero_hasta', 20, N),
    ('tipo_doc', 2, N),
    ('nro_doc', 20, N),
    ('nombre', 30, A),
    ('imp_total', 15, I),
    ('imp_tot_conc', 15, I),
    ('impto_liq_rni', 15, I),
    ('imp_op_ex', 15, I),
    ('impto_perc', 15, I),
    ('imp_iibb', 15, I),
    ('impto_perc_mun', 15, I),
    ('imp_internos', 15, I),
    ('imp_moneda_id', 3, A),
    ('imp_moneda_ctz', 10, I),
    ('cant_alicuota_iva', 1, N),
    ('codigo_operacion', 1, C),
    ('imp_trib', 15, I),
    ]

REGINFO_CV_VENTAS_CBTE_ALICUOTA = [
    ('tipo_cbte', 3, N),
    ('punto_vta', 5, N),
    ('cbt_numero', 20, N),
    ('imp_neto', 15, I),
    ('alicuota_iva', 4, I),
    ('impto_liq', 15, I),
    ]


def format_as_dict(format):
    return dict([(k[0], None) for k in format])


def leer_planilla(entrada, sep=','):
    "Convierte una planilla CSV a una lista de diccionarios [{'col': celda}]"

    items = []
    csv_reader = csv.reader(open(entrada), dialect='excel', delimiter=sep)
    for row in csv_reader:
        items.append(row)
    if len(items) < 2:
        raise RuntimeError('El archivo no tiene filas validos')
    if len(items[0]) < 2:
        raise RuntimeError('El archivo no tiene columnas (usar %s de separador)' % sep)
    cols = [str(it).strip() for it in items[0]]

    # armar diccionario por cada linea
    items = [dict([(cols[i],str(v).strip()) for i,v in enumerate(item)]) for item in items[1:]]

    return items


def leer_json(entrada="sired.json"):
    "Carga los datos en formato JSON [{'col': celda}]"

    import json
    items = json.load(open(entrada))

    return items


def grabar_json(salida="sired.json"):
    "Guarda los datos en formato JSON"

    import json
    json.dump(items, open(salida, "w"), sort_keys=True, indent=4)


def generar_encabezado(items):
    "Crear archivo de cabecera de facturas emitidas"

    periodo = items[0]['fecha_cbte'][:6]

    out = open("CABECERA_%s.txt" % periodo, "w")
    totales = format_as_dict(CAB_FAC_TIPO2)
    totales['periodo'] = periodo
    for key in IMPORTES:
        totales[key] = Decimal(0)

    for item in items:
        vals = format_as_dict(CAB_FAC_TIPO1)
        vals['fecha_anulacion'] = ''
        for k in item.keys():
            vals[k] = item[k]
            if k in totales and k in IMPORTES:
                totales[k] = totales[k] + Decimal(item[k])
        vals['tipo_reg'] = '1'
        vals['ctl_fiscal'] = item.get('ctl_fiscal', ' ')  # C para controlador
        vals['cbte_nro_reg'] = vals['cbt_numero']
        vals['cant_hojas'] = '01'
        vals['transporte'] = '0'
        vals['categoria'] = categorias[item['categoria'].lower()]
        if vals['imp_moneda_id'] is None:
            vals['imp_moneda_id'] = 'PES'
            vals['imp_moneda_ctz'] = '1.000'
        vals['alicuotas_iva'] = max(len(item.get('ivas', [])), 1)
        if vals['codigo_operacion'] is None:
            if int(item['tipo_cbte']) in (19, 20, 21):
                vals['codigo_operacion'] = 'E'
            else:
                vals['codigo_operacion'] = ' '
        if vals['imp_tot_conc'] is None:
            vals['imp_tot_conc'] = '0'
        s = escribir(vals, CAB_FAC_TIPO1)
        out.write(s)

    totales['tipo_reg'] = '2'
    totales['cant_reg_tipo_1'] = str(len(items))
    totales['cuit'] = CUIT
    s = escribir(totales, CAB_FAC_TIPO2)
    out.write(s)
    out.close()


def generar_detalle(items):
    "Crear archivo de detalle de facturas emitidas"

    periodo = items[0]['fecha_cbte'][:6]

    out = open("DETALLE_%s.txt" % periodo, "w")

    # recorro las facturas y detalles de artículos vendidos:
    for item in items:
        for it in item.get('detalles', [{}]):
            vals = format_as_dict(DETALLE)
            # datos generales de la factura:
            vals['tipo_reg'] = '1'
            for k in ('tipo_cbte', 'fecha_cbte', 'punto_vta', 'cbt_numero'):
                vals[k] = item[k]
            vals['cbte_nro_reg'] = item['cbt_numero']  # no hay varias hojas
            vals['ctl_fiscal'] = item.get('ctl_fiscal', ' ')  # C para controlador
            vals['anulacion'] = item.get('anulacion', ' ')
            # datos del artículo:
            vals['qty'] = it.get('qty', '1')  # cantidad
            vals['pro_umed'] = it.get('umed', '07')  # unidad de medida
            vals['pro_precio_uni'] = it.get('precio', item['imp_neto'])
            vals['imp_bonif'] = it.get('bonif', '0.00')
            vals['imp_ajuste'] = it.get('ajuste', '0.00')
            vals['imp_total'] = it.get('importe', '0.00')
            # iva
            if 'iva_id' in it and it['iva_id']:
                # mapear alicuota de iva según código usado en MTX
                iva_id = int(it['iva_id'])
                if iva_id in (1, 2):
                    alicuota = None
                else:
                    alicuota = {3: "0.00", 4: "10.5", 5: "21", 6: "27"}[iva_id]
                if alicuota is None:
                    vals['gravado'] = 'E'
                else:
                    vals['gravado'] = 'G'
                vals['alicuota_iva'] = alicuota or '0.00'
            else:
                # tomar datos generales:
                vals['alicuota_iva'] = (Decimal(item['imp_total'])/Decimal(item['imp_neto']) - 1) * 100
                if float(item.get('impto_liq', item.get('imp_iva', 0))) == 0:
                    vals['gravado'] = 'E'
                else:
                    vals['gravado'] = 'G'
            # diseño libre: código de barras y descripción:
            vals['codigo'] = it.get('codigo', '')
            vals['ds'] = it.get('ds', '')
            s = escribir(vals, DETALLE)
            out.write(s)

    out.close()


def generar_ventas(items):
    "Crear archivos de ventas (registros tipo 1 y tipo 2 totales)"

    periodo = items[0]['fecha_cbte'][:6]

    out = open("VENTAS_%s.txt" % periodo, "w")
    totales = format_as_dict(VENTAS_TIPO2)
    totales['periodo'] = periodo
    for key in IMPORTES:
        totales[key] = Decimal(0)

    # recorro las facturas e itero sobre los subtotales por alicuota de IVA:
    for item in items:
        ivas = item.get("ivas", [{}])
        for i, iva in enumerate(ivas):
            vals = format_as_dict(VENTAS_TIPO1)
            # datos generales de la factura:
            vals['tipo_reg'] = '1'
            # copio los campos que no varían para las distintas alicuotas de IVA
            for k, l, t in VENTAS_TIPO1[1:10] + VENTAS_TIPO1[21:30]:
                vals[k] = item.get(k)
            vals['fecha_anulacion'] = ''
            vals['ctl_fiscal'] = item.get('ctl_fiscal', ' ')  # C para controlador
            vals['anulacion'] = item.get('anulacion', ' ')
            vals['cbte_nro_reg'] = item['cbt_numero']
            vals['cant_hojas'] = '01'
            vals['transporte'] = '0'
            vals['categoria'] = categorias[item['categoria'].lower()]
            if vals['imp_moneda_id'] is None:
                vals['imp_moneda_id'] = 'PES'
                vals['imp_moneda_ctz'] = '1.000'
            # subtotales por alícuota de IVA
            if 'iva_id' in iva:
                # mapear alicuota de iva según código usado en MTX
                iva_id = int(iva['iva_id'])
                if iva_id == 1:
                    vals['imp_tot_conc'] = iva['base_imp']
                    alicuota = None
                elif iva_id == 2:
                    vals['imp_op_ex'] = iva['base_imp']
                    alicuota = None
                else:
                    alicuota = {3: "0.00", 4: "10.5", 5: "21", 6: "27"}[iva_id]
                    vals['imp_neto'] = iva['base_imp']
                    vals['impto_liq'] = iva['importe']
                vals['alicuota_iva'] = alicuota or '0.00'
            else:
                # tomar datos generales:
                vals['alicuota_iva'] = (Decimal(item['imp_total'])/Decimal(item['imp_neto']) - 1) * 100
                vals['alicuotas_iva'] = '01'
                if float(item.get('impto_liq', item.get('imp_iva', 0))) == 0:
                    vals['codigo_operacion'] = 'E'
                else:
                    vals['codigo_operacion'] = ' '
                if vals['imp_tot_conc'] is None:
                    vals['imp_tot_conc'] = '0'

            # acumulo los totales para el registro tipo 2
            for k in IMPORTES:
                totales[k] = totales[k] + Decimal(vals[k] or 0)

            # otros impuestos (TODO: recorrer tributos) solo ultimo registro:
            if len(ivas) == i-1:
                for k in ('impto_perc', 'imp_iibb', 'impto_perc_mun', 'imp_internos'):
                    if k in item:
                        vals[k] = item[k]
                        totales[k] = totales[k] + Decimal(vals[k] or 0)

            s = escribir(vals, VENTAS_TIPO1)
            out.write(s)

    totales['tipo_reg'] = '2'
    totales['cant_reg_tipo_1'] = str(len(items))
    totales['cuit'] = CUIT
    s = escribir(totales, VENTAS_TIPO2)
    out.write(s)
    out.close()


class SIRED():
    "Componente para Sistema Resúmen Electrónico de Datos RG1361/02 RG1579/03"

    _public_methods_ = ['CrearBD',
                        'CrearFactura',
                        'AgregarDetalleItem', 'AgregarIva', 'AgregarTributo',
                        'AgregarCmpAsoc', 'AgregarPermiso',
                        'AgregarDato',
                        'GuardarFactura', 'ObtenerFactura',
                        ]
    _public_attrs_ = ['InstallDir', 'Traceback', 'Excepcion', 'Version',
                     ]
    _readonly_attrs_ = _public_attrs_
    _reg_progid_ = "SIRED"
    _reg_clsid_ = "{3DC74AD5-939F-42AB-8381-FCA7AF783C77}"

    def __init__(self):
        self.db_path = os.path.join(self.InstallDir, "sired.db")
        self.Version = __version__
        # Abrir la base de datos
        crear = not os.path.exists(self.db_path)
        self.db = sqlite3.connect(self.db_path)
        self.db.row_factory = sqlite3.Row
        self.cursor = self.db.cursor()
        if crear:
            from formatos.formato_txt import ENCABEZADO, DETALLE, TRIBUTO, IVA, CMP_ASOC, PERMISO, DATO
            from formatos.formato_sql import esquema_sql
            tipos_registro =  [
                ('encabezado', ENCABEZADO),
                ('detalle', DETALLE),
                ('tributo', TRIBUTO),
                ('iva', IVA),
                ('cmp_asoc', CMP_ASOC),
                ('permiso', PERMISO),
                ('dato', DATO),
                ]
            for sql in esquema_sql(tipos_registro):
                self.cursor.execute(sql)

    def CrearFactura(self, concepto=1, tipo_doc=80, nro_doc="", tipo_cbte=1, punto_vta=0,
            cbte_nro=0, imp_total=0.00, imp_tot_conc=0.00, imp_neto=0.00,
            imp_iva=0.00, imp_trib=0.00, imp_op_ex=0.00, fecha_cbte="", fecha_venc_pago="",
            fecha_serv_desde=None, fecha_serv_hasta=None,
            moneda_id="PES", moneda_ctz="1.0000", cae="", fch_venc_cae="", id_impositivo='',
            nombre_cliente="", domicilio_cliente="", pais_dst_cmp=None,
            obs_comerciales="", obs_generales="", forma_pago="", incoterms="",
            idioma_cbte=7, motivos_obs="", descuento=0.0, email="",
            **kwargs
            ):
        "Creo un objeto factura (internamente)"
        fact = {'tipo_doc': tipo_doc, 'nro_doc':  nro_doc,
                'tipo_cbte': tipo_cbte, 'punto_vta': punto_vta,
                'cbte_nro': cbte_nro,
                'imp_total': imp_total, 'imp_tot_conc': imp_tot_conc,
                'imp_neto': imp_neto, 'imp_iva': imp_iva,
                'imp_trib': imp_trib, 'imp_op_ex': imp_op_ex,
                'fecha_cbte': fecha_cbte,
                'fecha_venc_pago': fecha_venc_pago,
                'moneda_id': moneda_id, 'moneda_ctz': moneda_ctz,
                'concepto': concepto,
                'nombre_cliente': nombre_cliente,
                'domicilio_cliente': domicilio_cliente,
                'pais_dst_cmp': pais_dst_cmp,
                'obs_comerciales': obs_comerciales,
                'obs_generales': obs_generales,
                'id_impositivo': id_impositivo,
                'forma_pago': forma_pago, 'incoterms': incoterms,
                'cae': cae, 'fecha_vto': fch_venc_cae,
                'motivos_obs': motivos_obs,
                'descuento': descuento,
                'email': email,
                'cbtes_asoc': [],
                'tributos': [],
                'ivas': [],
                'permisos': [],
                'detalles': [],
                'datos': [],
            }
        if fecha_serv_desde: fact['fecha_serv_desde'] = fecha_serv_desde
        if fecha_serv_hasta: fact['fecha_serv_hasta'] = fecha_serv_hasta
        self.factura = fact
        return True

    def EstablecerParametro(self, parametro, valor):
        "Modifico un parametro general a la factura (internamente)"
        self.factura[parametro] = valor
        return True

    def AgregarDato(self, campo, valor, pagina='T'):
        "Agrego un dato a la factura (internamente)"
        self.factura["datos"].append({'campo': campo, 'valor': valor, 'pagina': pagina})
        return True

    def AgregarDetalleItem(self, u_mtx, cod_mtx, codigo, ds, qty, umed, precio,
                    bonif, iva_id, imp_iva, importe, despacho,
                    dato_a=None, dato_b=None, dato_c=None, dato_d=None, dato_e=None):
        "Agrego un item a una factura (internamente)"
        ##ds = unicode(ds, "latin1") # convierto a latin1
        # Nota: no se calcula neto, iva, etc (deben venir calculados!)
        item = {
                'u_mtx': u_mtx,
                'cod_mtx': cod_mtx,
                'codigo': codigo,
                'ds': ds,
                'qty': qty,
                'umed': umed,
                'precio': precio,
                'bonif': bonif,
                'iva_id': iva_id,
                'imp_iva': imp_iva,
                'importe': importe,
                'despacho': despacho,
                'dato_a': dato_a,
                'dato_b': dato_b,
                'dato_c': dato_c,
                'dato_d': dato_d,
                'dato_e': dato_e,
                }
        self.factura['detalles'].append(item)
        return True

    def AgregarCmpAsoc(self, tipo=1, pto_vta=0, nro=0, **kwarg):
        "Agrego un comprobante asociado a una factura (interna)"
        cmp_asoc = {'cbte_tipo': tipo, 'cbte_punto_vta': pto_vta, 'cbte_nro': nro}
        self.factura['cbtes_asoc'].append(cmp_asoc)
        return True

    def AgregarTributo(self, tributo_id=0, desc="", base_imp=0.00, alic=0, importe=0.00, **kwarg):
        "Agrego un tributo a una factura (interna)"
        tributo = { 'tributo_id': tributo_id, 'desc': desc, 'base_imp': base_imp,
                    'alic': alic, 'importe': importe}
        self.factura['tributos'].append(tributo)
        return True

    def AgregarIva(self, iva_id=0, base_imp=0.0, importe=0.0, **kwarg):
        "Agrego un tributo a una factura (interna)"
        iva = { 'iva_id': iva_id, 'base_imp': base_imp, 'importe': importe }
        self.factura['ivas'].append(iva)
        return True

    def GuardarFactura(self):
        from formatos.formato_sql import escribir
        escribir([self.factura], self.db)
        return self.factura['id']

    def ActualizarFactura(self, id_factura):
        from formatos.formato_sql import modificar
        self.factura["id"] = id_factura
        modificar(self.factura, self.db)
        return True

    def ObtenerFactura(self, id_factura=None):
        from formatos.formato_sql import leer, max_id
        if not id_factura:
            id_factura = max_id(self.db)
        facts = list(leer(self.db, ids=[id_factura]))
        if facts:
            self.factura = facts[0]
        return True

    def Consultar(self, **kwargs):
        from formatos.formato_sql import leer
        return leer(self.db, **kwargs)


# busco el directorio de instalación (global para que no cambie si usan otra dll)
INSTALL_DIR = SIRED.InstallDir = get_install_dir()

if __name__ == '__main__':
    try:
        if hasattr(sys,"frozen") or False:
            p=os.path.dirname(os.path.abspath(sys.executable))
            os.chdir(p)
        ##sys.stdout = open("salida.txt", "a")
        entrada = {}
        for i, k in enumerate(('encabezados', 'detalles', 'ivas', 'tributos')):
            if len(sys.argv) > i+1:
                filename = sys.argv[i+1]
                if not filename.startswith("--") and os.path.exists(filename):
                    entrada[k] = filename
        if not entrada:
            entrada['encabezado'] = 'facturas3.csv'

        if '--prueba' in sys.argv:
            sired = SIRED()

            # creo una factura de ejemplo
            tipo_cbte = 2
            punto_vta = 4000
            fecha = datetime.datetime.now().strftime("%Y%m%d")
            concepto = 3
            tipo_doc = 80; nro_doc = "30000000007"
            cbte_nro = 12345678
            imp_total = "122.00"; imp_tot_conc = "3.00"
            imp_neto = "100.00"; imp_iva = "21.00"
            imp_trib = "1.00"; imp_op_ex = "2.00"; imp_subtotal = "100.00"
            fecha_cbte = fecha; fecha_venc_pago = fecha
            # Fechas del período del servicio facturado (solo si concepto = 1?)
            fecha_serv_desde = fecha; fecha_serv_hasta = fecha
            moneda_id = 'PES'; moneda_ctz = '1.000'
            obs_generales = "Observaciones Generales, texto libre"
            obs_comerciales = "Observaciones Comerciales, texto libre"

            nombre_cliente = 'Joao Da Silva'
            domicilio_cliente = 'Rua 76 km 34.5 Alagoas'
            pais_dst_cmp = 16
            id_impositivo = 'PJ54482221-l'
            moneda_id = '012'
            moneda_ctz = 0.5
            forma_pago = '30 dias'
            incoterms = 'FOB'
            idioma_cbte = 1
            motivo = "11"

            cae = None
            fch_venc_cae = None

            sired.CrearFactura(concepto, tipo_doc, nro_doc, tipo_cbte, punto_vta,
                cbte_nro, imp_total, imp_tot_conc, imp_neto,
                imp_iva, imp_trib, imp_op_ex, fecha_cbte, fecha_venc_pago,
                fecha_serv_desde, fecha_serv_hasta,
                moneda_id, moneda_ctz, cae, fch_venc_cae, id_impositivo,
                nombre_cliente, domicilio_cliente, pais_dst_cmp,
                obs_comerciales, obs_generales, forma_pago, incoterms,
                idioma_cbte, motivo)

            tipo = 91
            pto_vta = 2
            nro = 1234
            sired.AgregarCmpAsoc(tipo, pto_vta, nro)
            tipo = 5
            pto_vta = 2
            nro = 1234
            sired.AgregarCmpAsoc(tipo, pto_vta, nro)

            tributo_id = 99
            desc = 'Impuesto Municipal Matanza'
            base_imp = "100.00"
            alic = "1.00"
            importe = "1.00"
            sired.AgregarTributo(tributo_id, desc, base_imp, alic, importe)

            iva_id = 5 # 21%
            base_imp = 100
            importe = 21
            sired.AgregarIva(iva_id, base_imp, importe)

            u_mtx = 123456
            cod_mtx = 1234567890123
            codigo = "P0001"
            ds = "Descripcion del producto P0001\n" + "Lorem ipsum sit amet " * 10
            qty = 1.00
            umed = 7
            precio = 100.00
            bonif = 0.00
            iva_id = 5
            imp_iva = 21.00
            importe = 121.00
            despacho = u'Nº 123456'
            sired.AgregarDetalleItem(u_mtx, cod_mtx, codigo, ds, qty, umed,
                    precio, bonif, iva_id, imp_iva, importe, despacho)

            sired.AgregarDato("prueba", "1234")
            print("Prueba!")
            id_factura = sired.GuardarFactura()
            fact = sired.factura.copy()
            ok = sired.ObtenerFactura(id_factura)
            f = sired.factura

            # verificar que los datos se hayan grabado y leido correctamente:
            difs = []
            def cmp_dict(d1, d2, prefijo=None):
                global difs
                if difs is None:
                    difs = []
                for k in set(d1.keys() + d2.keys()):
                    if k in d1 and k in d2:
                        if isinstance(d1[k], list):
                            for i, (v1, v2) in enumerate(zip(d1[k], d2[k])):
                                cmp_dict(v1, v2, (k, i))
                        else:
                            if isinstance(d1[k], Decimal) or isinstance(d2[k], Decimal):
                                d1[k] = float(d1[k])
                                d2[k] = float(d2[k])
                            if isinstance(d1[k], long) or isinstance(d2[k], long):
                                d1[k] = long(d1[k])
                                d2[k] = long(d2[k])
                            if d1[k] != d2[k]:
                                difs.append(("Dif", prefijo, k, d1[k], d2[k]))
            cmp_dict(fact, f)
            for dif in difs:
                print(dif)

            sired.EstablecerParametro("cae", "61123022925855")
            sired.EstablecerParametro("fch_venc_cae", "20110320")
            sired.EstablecerParametro("motivo_obs", "")
            ok = sired.ActualizarFactura(id_factura)
            ok = sired.ObtenerFactura(id_factura)
            assert sired.factura["cae"] == "61123022925855"

            sys.exit(0)

        if '--leer' in sys.argv:
            if '--completar_padron' in sys.argv:
                from padron import PadronAFIP
                padron = PadronAFIP()
                padron.Conectar(trace="--trace" in sys.argv)
                from formatos import formato_txt
                formato = formato_txt.ENCABEZADO
                categorias_iva = dict([(int(v), k) for k, v in categorias.items()])

            else:
                formato = VENTAS_TIPO1

            claves = [clave for clave, pos, leng in formato if clave not in ('tipo','info_adic')]
            csv = csv.DictWriter(open("ventas.csv","wb"), claves, extrasaction='ignore')
            csv.writerow(dict([(k, k) for k in claves]))
            f = open("VENTAS.txt")
            for linea in f:
                if str(linea[0])=='2':
                    datos = leer(linea, REGINFO_CV_VENTAS_CBTE)

                    if '--completar_padron' in sys.argv:
                        cuit = datos['nro_doc']
                        print("Consultando AFIP online...", cuit,)
                        ok = padron.Consultar(cuit)
                        print(padron.direccion, padron.provincia)
                        datos["nombre_cliente"] = padron.denominacion.encode("latin1")
                        datos["domicilio_cliente"] = padron.direccion.encode("latin1")
                        datos["localidad_cliente"] = "%s (CP %s) " % (
                                                        padron.localidad.encode("latin1"),
                                                        padron.cod_postal.encode("latin1"))
                        datos["provincia_cliente"] = padron.provincia.encode("latin1")
                        datos['cbte_nro'] = datos['cbt_numero_desde']
                        #datos['id_impositivo'] = categorias_iva[int(datos['categoria'])]
                    csv.writerow(datos)
            f.close()
        else:
            # cargar datos desde planillas CSV separadas o JSON:
            if entrada['encabezados'].lower().endswith("csv"):
                facturas = items = leer_planilla(entrada['encabezados'], ";")

                # pre-procesar:
                for factura in facturas:
                    for k, v in factura.items():
                        # decodificar strings (evitar problemas unicode)
                        if isinstance(v, basestring):
                            if isinstance(v, str):
                                v = v.decode("latin1", "ignore")
                            factura[k] = unicodedata.normalize('NFKD', v).encode('ASCII', 'ignore')
                            print(k,factura[k])
                        # convertir tipos de datos desde los strings del CSV
                        if k.startswith("imp"):
                            factura[k] = float(v)
                        if k in ('cbt_desde', 'cbt_hasta', 'concepto',
                                 'punto_vta', 'tipo_cbte', 'tipo_doc',
                                 'nro_doc', 'cbt_numero'):
                            factura[k] = int(v)

                    alicuotas = {3:0, 4: 10.5, 5: 21., 6: 27}
                    ivas = {}
                    imp_iva = 0.00

                    ruta = os.path.dirname(entrada['encabezados'])
                    prefijos = ("%(tipo_cbte)02d%(cbt_numero)08d",
                               "%(tipo_cbte)02d%(cbt_numero)06d",
                               "%(tipo_cbte)02d%(punto_vta)04d%(cbt_numero)08d",
                               )
                    for prefijo in prefijos:
                        fn = os.path.join(ruta, "%s.csv" % (prefijo % factura))
                        print("Detalle: ", fn)
                        if os.path.exists(fn):
                            det = fn
                            print("encontrado!")
                            break
                    else:
                        if 'detalles' in entrada:
                            det = entrada['detalles']
                        else:
                            det = None

                    if det:
                        detalles = leer_planilla(det, ";")

                    for det in detalles:
                        iva_id = det.get('iva_id', 5)
                        if isinstance(det.get('ds'), str):
                            det['ds'] = det['ds'].decode("latin1", "ignore")
                        if 'ds' in det:
                            det['ds'] = unicodedata.normalize('NFKD', det['ds']).encode('ASCII', 'ignore')
                        print(det)
                        if iva_id:
                            iva_id = int(iva_id)
                            if iva_id not in ivas:
                                ivas[iva_id] = {"base_imp": 0, "importe": 0, "iva_id": iva_id}

                            importe = det.get('importe', det.get('total'))
                            if importe:
                                iva = det.get('imp_iva', None)
                                importe = round(float(importe.replace(",", ".")), 2)
                                if not iva is None:
                                    iva = round(float(iva.replace(",", ".")), 2)
                                # si el iva es incorrecto o no está, liquidar:
                                if not iva and iva_id > 3:
                                    # extraer IVA incluido factura B:
                                    if factura["tipo_cbte"] in (6, 7, 8):
                                        neto = round(importe / ((100 + alicuotas[iva_id]) / 100.), 2)
                                        iva = importe - neto
                                    else:
                                        neto = importe
                                        iva = round(neto * alicuotas[iva_id] /100., 2)
                                    print("importe iva calc:", importe, iva)
                                else:
                                    neto = importe
                                    # descontar IVA incluido factura B:
                                    if factura["tipo_cbte"] in (6, 7, 8):
                                        neto = neto - iva
                                imp_iva += iva
                                ivas[iva_id]['importe'] += iva
                                ivas[iva_id]['base_imp'] += neto
                                det['imp_iva'] = iva

                    # rearmar estructuras internas:
                    factura['detalles'] = detalles
                    factura['ivas'] = ivas.values()
                    factura['datos'] = []
                    factura['tributos'] = []
                    if 'imp_iva' not in factura or factura['imp_iva'] == "":
                        print("debe agregar el IVA total en el encabezado...")
                        factura['imp_iva'] = imp_iva
                    if 'cbt_numero' in factura:
                        factura['cbt_desde'] = factura['cbt_numero']
                        factura['cbt_hasta'] = factura['cbt_numero']
                    if 'nombre' in factura:
                        factura['nombre_cliente'] = factura['nombre']
                        factura['domicilio_cliente'] = factura['domicilio']
                    factura['cbte_nro'] = factura['cbt_desde']
                    if not 'concepto' in factura:
                        factura['concepto'] = 1

                    # limpio campos que no correspondan (productos vs servicios):
                    if factura['concepto'] == 1:
                        factura['fecha_venc_pago'] = None

            elif entrada['encabezados'].lower().endswith('.json'):
                items = leer_json(entrada['encabezados'])


            print("Generando encabezado...")
            generar_encabezado(items)
            print("Generando detalle...")
            generar_detalle(items)
            print("Generando ventas...")
            generar_ventas(items)
            if '--json' in sys.argv:
                grabar_json()

        print("Hecho.")
    except Exception as e:
        if '--debug' in sys.argv:
            raise
        print("Error: por favor corriga los datos y vuelva a intentar:")
        print(str(e))
        f = open("traceback.txt", "w+")
        import traceback
        traceback.print_exc(file=f)
        f.close()
    if '--debug' in sys.argv:
        raw_input("presione enter para continuar...")
    ##sys.stdout.close()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTIBILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.

"Módulo de Intefase para archivos de texto (bono fiscal)"

__author__ = "Mariano Reingart (mariano@nsis.com.ar)"
__copyright__ = "Copyright (C) 2009 Mariano Reingart"
__license__ = "GPL 3.0"
__version__ = "1.18"

import sys
import time
import traceback
from ConfigParser import SafeConfigParser

# revisar la instalación de pyafip.ws:
import wsaa
import wsbfe
from php import SimpleXMLElement, SoapClient, SoapFault, date

HOMO = False
DEBUG = False
XML = False
CONFIG_FILE = "rece.ini"

LICENCIA = """
receb.py: Interfaz de texto para generar Facturas Electrónicas Bienes de
Capital
Copyright (C) 2008/2009 Mariano Reingart reingart@gmail.com

Este progarma es software libre, se entrega ABSOLUTAMENTE SIN GARANTIA
y es bienvenido a redistribuirlo bajo la licencia GPLv3.

Para información adicional sobre garantía, soporte técnico comercial
e incorporación/distribución en programas propietarios ver PyAfipWs:
http://www.sistemasagiles.com.ar/trac/wiki/PyAfipWs
"""

# definición del formato del archivo de intercambio:
N = 'Numerico'
A = 'Alfanumerico'
I = 'Importe'
ENCABEZADO = [
    ('tipo_reg', 1, N),  # 0: encabezado
    ('fecha_cbte', 8, A),
    ('tipo_cbte', 2, N), ('punto_vta', 4, N),
    ('cbte_nro', 8, N),
    ('tipo_doc', 2, N), ('nro_doc', 11, N),
    ('imp_total', 15, I), ('imp_tot_conc', 15, I),
    ('imp_neto', 15, I), ('impto_liq', 15, I),
    ('impto_liq_rni', 15, I), ('imp_op_ex', 15, I),
    ('impto_perc', 15, I), ('imp_iibb', 15, I),
    ('impto_perc_mun', 15, I), ('imp_internos', 15, I),
    ('imp_moneda_id', 3, A),
    ('imp_moneda_ctz', 10, I),
    ('zona', 5, A),
    ('cae', 14, N), ('fecha_vto', 8, A),
    ('resultado', 1, A), ('obs', 2, A), ('reproceso', 1, A),
    ('id', 15, N),
    ]

DETALLE = [
    ('tipo_reg', 1, N),  # 1: detalle item
    ('ncm', 15, A),
    ('sec', 15, A),
    ('qty', 15, I),
    ('umed', 5, N),
    ('precio', 15, I),
    ('bonif', 15, I),
    ('imp_total', 15, I),
    ('iva_id', 5, N),
    ('ds', 200, A),
    ]


def leer(linea, formato):
    dic = {}
    comienzo = 1
    for (clave, longitud, tipo) in formato:
        valor = linea[comienzo-1:comienzo-1+longitud].strip()
        if tipo == N and valor:
            valor = str(int(valor))
        if tipo == I:
            if valor:
                valor = float("%s.%02d" % (int(valor[:-2]), int(valor[-2:])))
            else:
                valor = 0.00
        dic[clave] = valor

        comienzo += longitud
    return dic


translate_keys = {
    'ncm': 'Pro_codigo_ncm', 'bonif': 'Imp_bonif', 'precio': 'Pro_precio_uni',
    'sec': 'Pro_codigo_sec', 'ds': 'Pro_ds', 'umed': 'Pro_umed',
    'qty': 'Pro_qty', 'imp_moneda_id': 'Imp_moneda_Id'
}


def escribir(dic, formato):
    linea = " " * 335
    comienzo = 1
    for (clave, longitud, tipo) in formato:
        if clave.capitalize() in dic:
            clave = clave.capitalize()
        valor = str(dic.get(clave, ""))
        if valor == "" and clave in translate_keys:
            valor = str(dic.get(translate_keys[clave], ""))
        if tipo == N and valor and valor != "NULL":
            valor = ("%%0%dd" % longitud) % int(valor)
        elif tipo == I and valor:
            valor = ("%%0%dd" % longitud) % (float(valor)*100)
        else:
            valor = ("%%0%ds" % longitud) % valor
        linea = linea[:comienzo-1] + valor + linea[comienzo-1+longitud:]
        comienzo += longitud
    return linea + "\n"


def autenticar(cert, privatekey, url):
    tra = wsaa.create_tra("wsbfe")
    cms = wsaa.sign_tra(str(tra), str(cert), str(privatekey))
    xml = wsaa.call_wsaa(str(cms), url)
    ta = SimpleXMLElement(xml)
    token = str(ta.credentials.token)
    sign = str(ta.credentials.sign)
    return token, sign


def autorizar(client, token, sign, cuit, entrada, salida):
    # recupero el último número de transacción
    # id = wsbfe.ultnro(client, token, sign, cuit)
    detalles = []
    encabezado = {}
    for linea in entrada:
        if str(linea[0]) == '0':
            encabezado = leer(linea, ENCABEZADO)
        elif str(linea[0]) == '1':
            detalle = leer(linea, DETALLE)
            detalles.append(detalle)
        else:
            print("Tipo de registro incorrecto:", linea[0])

    if not encabezado['id'].strip():
        # TODO: habria que leer y/o grabar el id en el archivo
        # id += 1 # incremento el nº de transacción
        # Por el momento, el id se calcula con el tipo, pv y nº de comprobant
        i = long(encabezado['cbte_nro'])
        i += (
            int(encabezado['cbte_nro'])*10**4 + int(
                encabezado['punto_vta']))*10**8
        encabezado['id'] = i

    if not encabezado['zona'].strip():
        encabezado['zona'] = 0

    if 'testing' in sys.argv:
        ult_cbte, fecha, events = wsbfe.get_last_cmp(
            client, token, sign, cuit, punto_vta, tipo_cbte)
        encabezado['cbte_nro'] = ult_cbte + 1
        ult_id, events = wsbfe.get_last_id(client, token, sign, cuit)
        encabezado['id'] = ult_id + 1

    # encabezado['imp_moneda_ctz'] = 1.00
    factura = wsbfe.FacturaBF(**encabezado)
    for detalle in detalles:
        it = wsbfe.ItemBF(**detalle)
        factura.add_item(it, calc=False)

    if DEBUG:
        print(
            '\n'.join(
                ["%s='%s'" % (k, v) for k, v in factura.to_dict().items()])
        )
        print('id:', encabezado['id'])
    if not DEBUG or raw_input("Facturar?") == "S":
        auth, events = wsbfe.authorize(client, token, sign, cuit,
                                       id=encabezado['id'],
                                       factura=factura.to_dict())
        dic = factura.to_dict()
        dic.update(auth)
        escribir_factura(dic, salida)
        print(
            "ID:", dic['id'], "CAE:", dic['cae'], "Obs:", dic['obs'],
            "Reproceso:", dic['reproceso']
        )


def escribir_factura(dic, archivo):
    dic['tipo_reg'] = 0
    archivo.write(escribir(dic, ENCABEZADO))
    for it in dic['Items']:
        it['Item']['tipo_reg'] = 1
        archivo.write(escribir(it['Item'], DETALLE))


def depurar_xml(client):
    fecha = time.strftime("%Y%m%d%H%M%S")
    f = open("request-%s.xml" % fecha, "w")
    f.write(client.xml_request)
    f.close()
    f = open("response-%s.xml" % fecha, "w")
    f.write(client.xml_response)
    f.close()


if __name__ == "__main__":
    if '/ayuda' in sys.argv:
        print(LICENCIA)
        print("Opciones: ")
        print(" /ayuda: este mensaje")
        print(" /dummy: consulta estado de servidores")
        print(
            " /prueba: genera y autoriza una factura de prueba (no usar en \
            producción!)")
        print(" /ult: consulta último número de comprobante")
        print(" /id: consulta último ID")
        print(" /debug: modo depuración (detalla y confirma las operaciones)")
        print(
            " /formato: muestra el formato de los archivos de entrada/salida")
        print(
            " /get: recupera datos de un comprobante autorizado previamente \
            (verificación)")
        print(
            " /xml: almacena los requerimientos y respuestas XML (depuración)")
        print(
            "Ver rece.ini para parámetros de configuración (URL, \
            certificados, etc.)")
        sys.exit(0)

    config = SafeConfigParser()
    config.read(CONFIG_FILE)
    cert = config.get('WSAA', 'CERT')
    privatekey = config.get('WSAA', 'PRIVATEKEY')
    cuit = config.get('WSBFE', 'CUIT')
    entrada = config.get('WSBFE', 'ENTRADA')
    salida = config.get('WSBFE', 'SALIDA')

    if config.has_option('WSAA', 'URL') and not HOMO:
        wsaa_url = config.get('WS AA', 'URL')
    else:
        wsaa_url = wsaa.WSAAURL
    if config.has_option('WSBFE', 'URL') and not HOMO:
        wsbfe_url = config.get('WSBFE', 'URL')
    else:
        wsbfe_url = wsbfe.WSBFEURL

    if '/debug'in sys.argv:
        DEBUG = True

    if '/xml'in sys.argv:
        XML = True

    if DEBUG:
        print("wsaa_url %s\nwsbfe_url %s" % (wsaa_url, wsbfe_url))

    try:
        client = SoapClient(wsbfe_url, action=wsbfe.SOAP_ACTION,
                            namespace=wsbfe.SOAP_NS, trace=False,
                            exceptions=True)

        if '/dummy' in sys.argv:
            print("Consultando estado de servidores...")
            print(wsbfe.dummy(client))
            sys.exit(0)

        if '/formato' in sys.argv:
            print("Formato:")
            for msg, formato in [('Encabezado', ENCABEZADO),
                                 ('Detalle', DETALLE)]:
                comienzo = 1
                print("== %s ==" % msg)
                for (clave, longitud, tipo) in formato:
                    print(
                        " * Campo: %-20s Posición: %3d Longitud: %4d \
                        Tipo: %s" % (clave, comienzo, longitud, tipo)
                    )
                    comienzo += longitud
            sys.exit(0)

        # TODO: esto habría que guardarlo en un archivo y no tener que
        # autenticar cada vez
        token, sign = autenticar(cert, privatekey, wsaa_url)

        if '/prueba' in sys.argv or False:
            # generar el archivo de prueba para la próxima factura
            fecha = date('Ymd')
            tipo_cbte = 1
            punto_vta = 2
            ult_cbte, fecha, events = wsbfe.get_last_cmp(
                client, token, sign, cuit, punto_vta, tipo_cbte)
            ult_id, events = wsbfe.get_last_id(client, token, sign, cuit)

            f_entrada = open(entrada, "w")

            f = wsbfe.FacturaBF()
            f.punto_vta = punto_vta
            f.cbte_nro = ult_cbte+1
            f.imp_moneda_id = 'PES'
            f.fecha_cbte = date('Ymd')
            it = wsbfe.ItemBF(
                ncm='7308.10.00', sec='', ds='prueba', qty=2.0, precio=100.0,
                bonif=0.0, iva_id=5
            )
            f.add_item(it)
            it = wsbfe.ItemBF(
                ncm='7308.20.00', sec='', ds='prueba 2', qty=4.0, precio=50.0,
                bonif=10.0, iva_id=5)
            f.add_item(it)
            # print(f.to_dict())

            dic = f.to_dict()
            dic['id'] = ult_id
            escribir_factura(dic, f_entrada)
            f_entrada.close()

        if '/ult' in sys.argv:
            print("Consultar ultimo numero:")
            tipo_cbte = int(raw_input("Tipo de comprobante: "))
            punto_vta = int(raw_input("Punto de venta: "))
            ult_cbte, fecha, events = wsbfe.get_last_cmp(
                client, token, sign, cuit, tipo_cbte, punto_vta)
            print("Ultimo numero: ", ult_cbte)
            print("Fecha: ", fecha)
            depurar_xml(client)
            sys.exit(0)

        if '/id' in sys.argv:
            ult_id, events = wsbfe.get_last_id(client, token, sign, cuit)
            print("ID: ", fecha)
            depurar_xml(client)
            sys.exit(0)

        if '/get' in sys.argv:
            print("Recuperar comprobante:")
            tipo_cbte = int(raw_input("Tipo de comprobante: "))
            punto_vta = int(raw_input("Punto de venta: "))
            cbte_nro = int(raw_input("Numero de comprobante: "))
            cbt, events = wsbfe.get_cmp(
                client, token, sign, cuit, tipo_cbte, punto_vta, cbte_nro)
            for k, v in cbt.items():
                print("%s = %s" % (k, v))
            depurar_xml(client)
            sys.exit(0)

        f_entrada = f_salida = None
        try:
            f_entrada = open(entrada, "r")
            f_salida = open(salida, "w")
            try:
                autorizar(client, token, sign, cuit, f_entrada, f_salida)
            except (SoapFault, wsbfe.BFEError) as e:
                XML = True
                raise
        finally:
            if f_entrada is not None:
                f_entrada.close()
            if f_salida is not None:
                f_salida.close()
            if XML:
                depurar_xml(client)
        sys.exit(0)

    except SoapFault as e:
        print(e.faultcode, e.faultstring.encode("ascii", "ignore"))
        sys.exit(3)
    except wsbfe.BFEError as e:
        print(e.code, e.msg.encode("ascii", "ignore"))
        sys.exit(4)
    except Exception as e:
        print(unicode(e).encode("ascii", "ignore"))
        if DEBUG:
            raise
        sys.exit(5)



#!/usr/bin/python
# -*- coding: utf-8 -*-
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTIBILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.

"""Módulo para interfaz Depositario Fiel web service wDigDepFiel de AFIP
"""

__author__ = "Mariano Reingart <reingart@gmail.com>"
__copyright__ = "Copyright (C) 2010 Mariano Reingart"
__license__ = "LGPL 3.0"
__version__ = "1.01"

LICENCIA = """
wdigdepfiel.py: Interfaz para Digitalizacion Depositario Fiel AFIP
Copyright (C) 2010 Mariano Reingart reingart@gmail.com

Este progarma es software libre, se entrega ABSOLUTAMENTE SIN GARANTIA
y es bienvenido a redistribuirlo bajo la licencia GPLv3.

Para información adicional sobre garantía, soporte técnico comercial
e incorporación/distribución en programas propietarios ver PyAfipWs:
http://www.sistemasagiles.com.ar/trac/wiki/PyAfipWs
"""

import os, sys, time
from php import date, SimpleXMLElement, SoapClient, SoapFault

WSDDFURL = "https://testdia.afip.gov.ar/Dia/Ws/wDigDepFiel/wDigDepFiel.asmx"
SOAP_ACTION = 'ar.gov.afip.dia.serviciosWeb.wDigDepFiel/'
SOAP_NS = 'ar.gov.afip.dia.serviciosWeb.wDigDepFiel'

DEBUG = True
XML = False
HOMO = True


def dummy(client):
    "Metodo dummy para verificacion de funcionamiento"
    response = client.Dummy()
    result = response.DummyResult
    appserver = dbserver = authserver = None
    try:
        appserver = str(result.appserver)
        dbserver = str(result.dbserver)
        authserver = str(result.authserver)
    except (RuntimeError, IndexError, AttributeError) as e:
        pass
    return {'appserver': appserver,
            'dbserver': dbserver,
            'authserver': authserver}

def aviso_recep_acept(client, token, sign, cuit, tipo_agente, rol,
                      nro_legajo, cuit_declarante, cuit_psad, cuit_ie,
                          codigo, fecha_hora_acept, ticket):
    "Aviso de recepcion y aceptacion."
    response = client.AvisoRecepAcept(
                autentica=dict(Cuit=cuit, Token=token, Sign=sign, TipoAgente=tipo_agente, Rol=rol),
                nroLegajo=nro_legajo,
                cuitDeclarante=cuit_declarante,
                cuitPSAD=cuit_psad,
                cuitIE=cuit_ie,
                codigo=codigo,
                fechaHoraAcept=fecha_hora_acept,
                ticket=ticket,
                )
    result = response.AvisoRecepAceptResult
    return str(result.codError), str(result.descError)


def aviso_digit(client, token, sign, cuit, tipo_agente, rol,
                nro_legajo, cuit_declarante, cuit_psad, cuit_ie, cuit_ata,
                codigo, url, familias, ticket, hashing, cantidad_total):
    "Aviso de digitalizacion."
    response = client.AvisoDigit(
                autentica=dict(Cuit=cuit, Token=token, Sign=sign, TipoAgente=tipo_agente, Rol=rol),
                nroLegajo=nro_legajo,
                cuitDeclarante=cuit_declarante,
                cuitPSAD=cuit_psad,
                cuitIE=cuit_ie,
                cuitATA=cuit_ata,
                codigo=codigo,
                url=url,
                familias=familias,
                ticket=ticket,
                hashing=hashing,
                cantidadTotal=cantidad_total,
                )
    result = response.AvisoDigitResult
    return str(result.codError), str(result.descError)


if __name__ == '__main__':
    if '--ayuda' in sys.argv:
        print(LICENCIA)
        print(AYUDA)
        sys.exit(0)

    import csv
    import traceback
    import datetime

    import wsaa

    try:

        if "--version" in sys.argv:
            print("Versión: ", __version__)

        CERT='reingart.crt'
        PRIVATEKEY='reingart.key'
        # obteniendo el TA
        TA = "wsddf-ta.xml"
        if not os.path.exists(TA) or os.path.getmtime(TA)+(60*60*5)<time.time():
            tra = wsaa.create_tra(service="wDigDepFiel")
            cms = wsaa.sign_tra(tra,CERT,PRIVATEKEY)
            ta_string = wsaa.call_wsaa(cms)
            open(TA,"w").write(ta_string)
        ta_string=open(TA).read()
        ta = SimpleXMLElement(ta_string)
        token = str(ta.credentials.token)
        sign = str(ta.credentials.sign)
        # fin TA

        # cliente soap del web service
        client = SoapClient(WSDDFURL,
            action = SOAP_ACTION,
            namespace = SOAP_NS, exceptions = True,
            trace = True, ns = 'ar', soap_ns='soap')

        if '--dummy' in sys.argv:
            ret = dummy(client)
            print('\n'.join(["%s: %s" % it for it in ret.items()]))

        # ejemplos aviso recep acept (prueba):

        cuit = 20267565393
        tipo_agente = 'DESP' # 'DESP'
        rol = 'EXTE'
        nro_legajo = '0'*16 # '1234567890123456'
        cuit_declarante = cuit_psad = cuit_ie = cuit
        codigo = '000' # carpeta completa, '0001' carpeta adicional
        fecha_hora_acept = datetime.datetime.now().isoformat()
        ticket = '1234'
        r = aviso_recep_acept(client, token, sign, cuit, tipo_agente, rol,
                              nro_legajo, cuit_declarante, cuit_psad, cuit_ie,
                              codigo, fecha_hora_acept, ticket)
        print(r)


        # ejemplos aviso digit (prueba):

        cuit = 20267565393
        tipo_agente = 'DESP' # 'DESP'
        rol = 'EXTE'
        nro_legajo = '0'*16 # '1234567890123456'
        cuit_declarante = cuit_psad = cuit_ie = cuit_ata = cuit
        codigo = '000' # carpeta completa, '0001' carpeta adicional
        ticket = '1234'
        url = 'http://www.example.com'
        hashing = 'db1491eda47d78532cdfca19c62875aade941dc2'
        familias = [ {'Familia': {'codigo': '02', 'cantidad': 1}}, {'Familia': {'codigo': '03', 'cantidad': 3}}, ]
        cantidad_total = 4
        r = aviso_digit(client, token, sign, cuit, tipo_agente, rol,
                        nro_legajo, cuit_declarante, cuit_psad, cuit_ie, cuit_ata,
                        codigo, url, familias, ticket, hashing, cantidad_total)
        print(r)
        print("hecho.")

    except SoapFault as e:
        print("Falla SOAP:", e.faultcode, e.faultstring.encode("ascii","ignore"))
        sys.exit(3)
    except Exception as e:
        print(unicode(e).encode("ascii","ignore"))
        if DEBUG:
            raise
        sys.exit(5)



#!/usr/bin/python
# -*- coding: utf-8 -*-
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by the
# Free Software Foundation; either version 3, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTIBILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.

# Based on MultipartPostHandler.py (C) 02/2006 Will Holcomb <wholcomb@gmail.com>
# Ejemplos iniciales gracias a "Matias Gieco matigro@gmail.com"

"Módulo para obtener remito electrónico automático (COT)"

__author__ = "Mariano Reingart (reingart@gmail.com)"
__copyright__ = "Copyright (C) 2010 Mariano Reingart"
__license__ = "LGPL 3.0"
__version__ = "1.02h"

import os
import sys
import traceback
from py3simplesoap.simplexml import SimpleXMLElement

from .utils import WebClient

import logging

log = logging.getLogger()

HOMO = False
CACERT = "conf/arba.crt"   # establecimiento de canal seguro (en producción)

##URL = "https://cot.ec.gba.gob.ar/TransporteBienes/SeguridadCliente/"
#"presentarRemitos.do"
# Nuevo servidor para el "Remito Electrónico Automático"
URL = "http://cot.test.arba.gov.ar/TransporteBienes/SeguridadCliente/"
"presentarRemitos.do"  # testing
#URL = "https://cot.arba.gov.ar/TransporteBienes/SeguridadCliente/"
#"presentarRemitos.do"  # prod.


class COT:
    "Interfaz para el servicio de Remito Electronico ARBA"
    _public_methods_ = ['Conectar', 'PresentarRemito', 'LeerErrorValidacion',
                        'LeerValidacionRemito',
                        'AnalizarXml', 'ObtenerTagXml']
    _public_attrs_ = [
        'Usuario', 'Password', 'XmlResponse',
        'Version', 'Excepcion', 'Traceback', 'InstallDir',
        'CuitEmpresa', 'NumeroComprobante', 'CodigoIntegridad',
        'NombreArchivo', 'TipoError', 'CodigoError', 'MensajeError',
        'NumeroUnico', 'Procesado',
    ]

    _reg_progid_ = "COT"
    _reg_clsid_ = "{7518B2CF-23E9-4821-BC55-D15966E15620}"

    Version = "%s %s" % (__version__, HOMO and 'Homologación' or '')

    def __init__(self):
        self.Usuario = self.Password = None
        self.TipoError = self.CodigoError = self.MensajeError = ""
        self.LastID = self.LastCMP = self.CAE = self.CAEA = \
            self.Vencimiento = ''
        self.InstallDir = INSTALL_DIR
        self.client = None
        self.xml = None
        self.limpiar()

    def limpiar(self):
        self.remitos = []
        self.errores = []
        self.XmlResponse = ""
        self.Excepcion = self.Traceback = ""
        self.TipoError = self.CodigoError = self.MensajeError = ""
        self.CuitEmpresa = self.NumeroComprobante = ""
        self.NombreArchivo = self.CodigoIntegridad = ""
        self.NumeroUnico = self.Procesado = ""

    def Conectar(
        self, url=None, proxy="", wrapper=None, cacert=None, trace=False
    ):
        if HOMO or not url:
            url = URL
        self.client = WebClient(location=url, trace=trace, cacert=cacert)

    def PresentarRemito(self, filename, testing=""):
        self.limpiar()
        try:
            if not os.path.exists(filename):
                self.Excepcion = "Archivo no encontrado: %s" % filename
                return False

            archivo = open(filename, "rb")
            if not testing:
                response = self.client(
                    user=self.Usuario, password=self.Password, file=archivo
                )
            else:
                response = open(testing).read()
            self.XmlResponse = response
            self.xml = SimpleXMLElement(response)
            if 'tipoError' in self.xml:
                self.TipoError = str(self.xml.tipoError)
                self.CodigoError = str(self.xml.codigoError)
                self.MensajeError = str(self.xml.mensajeError)\
                    .decode('latin1').encode("ascii", "replace")
            if 'cuitEmpresa' in self.xml:
                self.CuitEmpresa = str(self.xml.cuitEmpresa)
                self.NumeroComprobante = str(self.xml.numeroComprobante)
                self.NombreArchivo = str(self.xml.nombreArchivo)
                self.CodigoIntegridad = str(self.xml.codigoIntegridad)
                if 'validacionesRemitos' in self.xml:
                    for remito in self.xml.validacionesRemitos.remito:
                        d = {
                            'NumeroUnico': str(remito.numeroUnico),
                            'Procesado': str(remito.procesado),
                            'Errores': [],
                            }
                        if 'errores' in remito:
                            for error in remito.errores.error:
                                d['Errores'].append(
                                    (
                                        str(error.codigo),
                                        str(error.descripcion)
                                        .decode('latin1')
                                        .encode("ascii", "replace")
                                    )
                                )
                        self.remitos.append(d)
                    # establecer valores del primer remito (sin eliminarlo)
                    self.LeerValidacionRemito(pop=False)
            return True
        except Exception as e:
            log.warn(e)
            ex = traceback.format_exception(
                sys.exc_info()[0], sys.exc_info()[1], sys.exc_info()[2]
            )
            self.Traceback = ''.join(ex)
            try:
                self.Excepcion = traceback.format_exception_only(
                    sys.exc_info()[0], sys.exc_info()[1]
                )[0]
            except:
                self.Excepcion = "<no disponible>"
            return False

    def LeerValidacionRemito(self, pop=True):
        "Leeo el próximo remito"
        # por compatibilidad hacia atras, la primera vez no remueve de la lista
        # (llamado de PresentarRemito con pop=False)
        if self.remitos:
            remito = self.remitos[0]
            if pop:
                del self.remitos[0]
            self.NumeroUnico = remito['NumeroUnico']
            self.Procesado = remito['Procesado']
            self.errores = remito['Errores']
            return True
        else:
            self.NumeroUnico = ""
            self.Procesado = ""
            self.errores = []
            return False

    def LeerErrorValidacion(self):
        if self.errores:
            error = self.errores.pop()
            self.TipoError = ""
            self.CodigoError = error[0]
            self.MensajeError = error[1]
            return True
        else:
            self.TipoError = ""
            self.CodigoError = ""
            self.MensajeError = ""
            return False

    def AnalizarXml(self, xml=""):
        "Analiza un mensaje XML (por defecto la respuesta)"
        try:
            if not xml:
                xml = self.XmlResponse
            self.xml = SimpleXMLElement(xml)
            return True
        except Exception as e:
            self.Excepcion = "%s" % (e)
            return False

    def ObtenerTagXml(self, *tags):
        "Busca en el Xml analizado y devuelve el tag solicitado"
        # convierto el xml a un objeto
        try:
            if self.xml:
                xml = self.xml
                # por cada tag, lo busco segun su nombre o posición
                for tag in tags:
                    xml = xml(tag)  # atajo a getitem y getattr
                # vuelvo a convertir a string el objeto xml encontrado
                return str(xml)
        except Exception as e:
            self.Excepcion = "%s" % (e)

# busco el directorio de instalación (global para que no cambie si usan
# otra dll)
if not hasattr(sys, "frozen"):
    basepath = __file__
elif sys.frozen == 'dll':
    import win32api
    basepath = win32api.GetModuleFileName(sys.frozendllhandle)
else:
    basepath = sys.executable
INSTALL_DIR = os.path.dirname(os.path.abspath(basepath))


if __name__ == "__main__":

    if "--register" in sys.argv or "--unregister" in sys.argv:
        import win32com.server.register
        win32com.server.register.UseCommandLine(COT)
        sys.exit(0)
    elif len(sys.argv) < 4:
        print("Se debe especificar el nombre de archivo, usuario y clave como\
                argumentos!")
        sys.exit(1)

    cot = COT()
    filename = sys.argv[1]      # TB_20111111112_000000_20080124_000001.txt
    cot.Usuario = sys.argv[2]   # 20267565393
    cot.Password = sys.argv[3]  # 23456

    if '--testing' in sys.argv:
        test_response = "cot_response_multiple_errores.xml"
        #test_response = "cot_response_2_errores.xml"
        #test_response = "cot_response_3_sinerrores.xml"
    else:
        test_response = ""

    if not HOMO:
        for i, arg in enumerate(sys.argv):
            if arg.startswith("--prod"):
                URL = URL.replace("http://cot.test.arba.gov.ar",
                                  "https://cot.arba.gov.ar")
                print("Usando URL:", URL)
                break
            if arg.startswith("https"):
                URL = arg
                print("Usando URL:", URL)
                break

    cot.Conectar(URL, trace='--trace' in sys.argv, cacert=CACERT)
    cot.PresentarRemito(filename, testing=test_response)

    if cot.Excepcion:
        print("Excepcion:", cot.Excepcion)
        print("Traceback:", cot.Traceback)

    # datos generales:
    print("CUIT Empresa:", cot.CuitEmpresa)
    print("Numero Comprobante:", cot.NumeroComprobante)
    print("Nombre Archivo:", cot.NombreArchivo)
    print("Codigo Integridad:", cot.CodigoIntegridad)

    print(
        "Error General:", cot.TipoError, "|", cot.CodigoError,
        "|", cot.MensajeError
    )

    # recorro los remitos devueltos e imprimo sus datos por cada uno:
    while cot.LeerValidacionRemito():
        print("Numero Unico:", cot.NumeroUnico)
        print("Procesado:", cot.Procesado)
        while cot.LeerErrorValidacion():
            print(
                "Error Validacion:", "|", cot.CodigoError,
                "|", cot.MensajeError
            )

    # Ejemplos de uso ObtenerTagXml
    if False:
        print("cuit", cot.ObtenerTagXml('cuitEmpresa'))
        print(
            "p0",
            cot.ObtenerTagXml('validacionesRemitos', 'remito', 0, 'procesado')
        )
        print(
            "p1",
            cot.ObtenerTagXml('validacionesRemitos', 'remito', 1, 'procesado')
        )



#!/usr/bin/python
# -*- coding: utf-8 -*-
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by the
# Free Software Foundation; either version 3, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTIBILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.

"Módulo con funciones y objetos para compatibilidad con PHP"

__author__ = "Mariano Reingart (mariano@nsis.com.ar)"
__copyright__ = "Copyright (C) 2008 Mariano Reingart"
__license__ = "LGPL 3.0"
__version__ = "1.0"

import datetime
import sys
import time
import xml.dom.minidom
import httplib2
from .soap import SoapFault, SoapClient, parse_proxy, SimpleXMLElement


def date(fmt=None, timestamp=None):
    "Manejo de fechas (simil PHP)"
    if fmt == 'U':  # return timestamp
        t = datetime.datetime.now()
        return int(time.mktime(t.timetuple()))
    if fmt == 'c':  # return isoformat
        d = datetime.datetime.fromtimestamp(timestamp)
        return d.isoformat()
    if fmt == 'Ymd':
        d = datetime.datetime.now()
        return d.strftime("%Y%m%d")



#!/usr/bin/python
# -*- coding: utf-8 -*-

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; either version 3, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTIBILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.

"Py2Exe extension to build NSIS Installers"

# Based on py2exe/samples/extending/setup.py:
#   "A setup script showing how to extend py2exe."
#   Copyright (c) 2000-2008 Thomas Heller, Mark Hammond, Jimmy Retzlaff

__author__ = "Mariano Reingart (reingart@gmail.com)"
__copyright__ = "Copyright (C) 2011 Mariano Reingart"
__license__ = "GPL 3.0"

import os
import sys
from py2exe.build_exe import py2exe


nsi_base_script = r"""\
; base.nsi

; WARNING: This script has been created by py2exe. Changes to this script
; will be overwritten the next time py2exe is run!

XPStyle on

Page license
Page directory
;Page components
Page instfiles

RequestExecutionLevel admin

LoadLanguageFile "${NSISDIR}\Contrib\Language files\English.nlf"
LoadLanguageFile "${NSISDIR}\Contrib\Language files\Spanish.nlf"

# set license page
LicenseText ""
LicenseData "licencia.txt"
LicenseForceSelection checkbox

; use the default string for the directory page.
DirText ""

Name "%(description)s"
OutFile "%(out_file)s"
;SetCompress off ; disable compression (testing)
SetCompressor /SOLID lzma
;InstallDir %(install_dir)s
InstallDir $PROGRAMFILES\%(install_dir)s

InstallDirRegKey HKLM "Software\%(reg_key)s" "Install_Dir"

VIProductVersion "%(product_version)s"
VIAddVersionKey /LANG=${LANG_ENGLISH} "ProductName" "%(name)s"
VIAddVersionKey /LANG=${LANG_ENGLISH} "FileDescription" "%(description)s"
VIAddVersionKey /LANG=${LANG_ENGLISH} "CompanyName" "%(company_name)s"
VIAddVersionKey /LANG=${LANG_ENGLISH} "FileVersion" "%(product_version)s"
VIAddVersionKey /LANG=${LANG_ENGLISH} "LegalCopyright" "%(copyright)s"
;VIAddVersionKey /LANG=${LANG_ENGLISH} "InternalName" "FileSetup.exe"

Section %(name)s
    ; uninstall old version

    ReadRegStr $R0 HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\%(reg_key)s" "UninstallString"
    StrCmp $R0 "" notistalled
    ExecWait '$R0 /S _?=$INSTDIR'

    ; clean up executable files
    Delete $INSTDIR\*.pyc
    Delete $INSTDIR\*.pyd
    Delete $INSTDIR\*.pyo
    Delete $INSTDIR\*.dll
    Delete $INSTDIR\*.exe

    ; clean up subdirectories
    FindFirst $0 $1 $INSTDIR\*
    cleanup_loop:
        StrCmp $1 "" cleanup_done
        StrCmp $1 "." cleanup_next
        StrCmp $1 ".." cleanup_next
        StrCmp $1 "conf" cleanup_next
        StrCmp $1 "datos" cleanup_next
        StrCmp $1 "plantillas" cleanup_next
        IfFileExists $INSTDIR\$1\*.* 0 +2
            RMDir /r $INSTDIR\$1
    cleanup_next:
        FindNext $0 $1
        Goto cleanup_loop
    cleanup_done:
    FindClose $0


notistalled:

    SectionIn RO
    SetOutPath $INSTDIR
    File /r dist\*.*
    IfFileExists $INSTDIR\\conf\\rece.ini 0 +3
        IfFileExists $INSTDIR\\rece.ini +2 0
        CopyFiles $INSTDIR\\conf\\rece.ini $INSTDIR\\rece.ini
    IfFileExists $INSTDIR\\conf\\reingart.crt 0 +3
        IfFileExists $INSTDIR\\reingart.crt +2 0
        CopyFiles $INSTDIR\\conf\\reingart.crt $INSTDIR\\reingart.crt
    IfFileExists $INSTDIR\\conf\\reingart.key 0 +3
        IfFileExists $INSTDIR\\reingart.key +2 0
        CopyFiles $INSTDIR\\conf\\reingart.key $INSTDIR\\reingart.key
    WriteRegStr HKLM SOFTWARE\%(reg_key)s "Install_Dir" "$INSTDIR"
    ; Write the uninstall keys for Windows
    WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\%(reg_key)s" "DisplayName" "%(description)s (solo eliminar)"
    WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\%(reg_key)s" "UninstallString" "$INSTDIR\Uninst.exe"
    WriteUninstaller "Uninst.exe"
    %(install_vcredist)s
    ;To Register a DLL
    %(register_com_servers_dll)s
    %(register_com_servers_exe)s
    ;create start-menu items
    IfFileExists $INSTDIR\\pyrece.exe 0 +4
        CreateDirectory "$SMPROGRAMS\%(name)s"
        CreateShortCut "$SMPROGRAMS\%(name)s\PyRece.lnk" "$INSTDIR\pyrece.exe" "" "$INSTDIR\pyrece.exe" 0
        CreateShortCut "$SMPROGRAMS\%(name)s\Designer.lnk" "$INSTDIR\designer.exe" "" "$INSTDIR\designer.exe" 0
        ;CreateShortCut "$SMPROGRAMS\%(name)s\Uninstall.lnk" "$INSTDIR\Uninst.exe" "" "$INSTDIR\Uninst.exe" 0
    IfFileExists $INSTDIR\\factura.exe 0 +3
        CreateDirectory "$SMPROGRAMS\%(name)s"
        CreateShortCut "$SMPROGRAMS\%(name)s\PyFactura.lnk" "$INSTDIR\factura.exe" "" "$INSTDIR\factura.exe" 0

SectionEnd

Section "Uninstall"
    ;To Unregister a DLL
    %(unregister_com_servers_dll)s
    %(unregister_com_servers_exe)s
    ;Delete Files

    ;Delete Uninstaller And Unistall Registry Entries
    Delete "$INSTDIR\Uninst.exe"
    DeleteRegKey HKEY_LOCAL_MACHINE "SOFTWARE\%(reg_key)s"
    DeleteRegKey HKEY_LOCAL_MACHINE "SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\%(reg_key)s"

SectionEnd

;--------------------------------

Function .onInit

    IfSilent nolangdialog

    ;Language selection dialog

    Push ""
    Push ${LANG_ENGLISH}
    Push English
    Push ${LANG_SPANISH}
    Push Spanish
    Push A ; A means auto count languages
           ; for the auto count to work the first empty push (Push "") must remain
    LangDLL::LangDialog "Installer Language" "Please select the language of the installer"

    Pop $LANGUAGE
    StrCmp $LANGUAGE "cancel" 0 +2
        Abort

nolangdialog:

FunctionEnd

"""

register_com_server_dll = """\
    RegDLL "$INSTDIR\%s"
"""
register_com_server_exe = """\
    ExecWait '%s /register'
"""
unregister_com_server_dll = """\
    UnRegDLL "$INSTDIR\%s"
"""
unregister_com_server_exe = """\
    ExecWait '%s /unregister'
"""

install_vcredist = r"""
    ReadRegStr $0 HKLM "SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\{FF66E9F6-83E7-3A3E-AF14-8DE9A809A6A4}" "DisplayName"
    StrCmp $0 "Microsoft Visual C++ 2008 Redistributable - x86 9.0.21022"  vcredist_ok vcredist_install

    vcredist_install:
    File "vcredist_x86.exe"
    DetailPrint "Installing Microsoft Visual C++ 2008 Redistributable"
    ExecWait '"$INSTDIR\vcredist_x86.exe" /q' $0
    Delete $INSTDIR\vcredist_x86.exe
    vcredist_ok:

"""


class build_installer(py2exe):
    # This class first builds the exe file(s), then creates a Windows installer.
    # You need NSIS (Nullsoft Scriptable Install System) for it.

    def run(self):
        # Clean up
        os.system("del /S /Q dist")
        # First, let py2exe do it's work.
        py2exe.run(self)

        lib_dir = self.lib_dir
        dist_dir = self.dist_dir
        comserver_files = self.comserver_files
        metadata = self.distribution.metadata

        # create the Installer, using the files py2exe has created.
        script = NSISScript(metadata,
                            lib_dir,
                            dist_dir,
                            self.windows_exe_files,
                            self.lib_files,
                            comserver_files)
        print("*** creating the nsis script***")
        script.create()
        print("*** compiling the nsis script***")
        script.compile()
        # Note: By default the final setup.exe will be in an Output
        # subdirectory.


class NSISScript:

    def __init__(self,
                 metadata,
                 lib_dir,
                 dist_dir,
                 windows_exe_files=[],
                 lib_files=[],
                 comserver_files=[]):
        self.lib_dir = lib_dir
        self.dist_dir = dist_dir
        if not self.dist_dir[-1] in "\\/":
            self.dist_dir += "\\"
        self.name = metadata.get_name()
        self.description = metadata.get_name()
        self.version = metadata.get_version()
        self.copyright = metadata.get_author()
        self.url = metadata.get_url()
        self.windows_exe_files = [self.chop(p) for p in windows_exe_files]
        self.lib_files = [self.chop(p) for p in lib_files]
        self.comserver_files_exe = [self.chop(p)
                                    for p in comserver_files if p.lower().endswith(".exe")]
        self.comserver_files_dll = [self.chop(p)
                                    for p in comserver_files if p.lower().endswith(".dll")]

    def chop(self, pathname):
        global install_vcredist
        # print(pathname, self.dist_dir)
        # assert pathname.startswith(self.dist_dir)
        if 'Microsoft.VC90.CRT.manifest' in pathname:
            # clean redistributable instructions (DLL files already included)
            install_vcredist = ""
        return pathname[len(self.dist_dir):]

    def create(self, pathname="base.nsi"):
        self.pathname = pathname
        ofi = self.file = open(pathname, "w")
        ver = self.version
        if "-" in ver:
            ver = ver[:ver.index("-")]
        rev = self.version.endswith("-full") and ".1" or ".0"
        ver = [c in '0123456789.' and c or ".%s" %
               (ord(c) - 96) for c in ver] + [rev]
        ofi.write(nsi_base_script % {
            'name': self.name,
            'description': "%s version %s" % (self.description, self.version),
            'product_version': ''.join(ver),
            'company_name': self.url,
            'copyright': self.copyright,
            'install_dir': self.name,
            'reg_key': self.name,
            'out_file': "%s-%s.exe" % (self.name, self.version if len(self.version) < 128 else (self.version[:14] + self.version[-5:])),
            'install_vcredist': install_vcredist if sys.version_info > (2, 7) else "",
            'register_com_servers_exe': ''.join([register_com_server_exe % comserver for comserver in self.comserver_files_exe]),
            'register_com_servers_dll': ''.join([register_com_server_dll % comserver for comserver in self.comserver_files_dll]),
            'unregister_com_servers_exe': ''.join([unregister_com_server_exe % comserver for comserver in self.comserver_files_exe]),
            'unregister_com_servers_dll': ''.join([unregister_com_server_dll % comserver for comserver in self.comserver_files_dll]),
        })

    def compile(self, pathname="base.nsi"):
        os.startfile(pathname, 'compile')


class Target():

    def __init__(self, module, **kw):
        self.__dict__.update(kw)
        # for the version info resources (Properties -- Version)
        # convertir 1.21a en 1.21.1
        self.version = module.__version__[
            :-1] + "." + str(ord(module.__version__[-1]) - 96)
        self.description = module.__doc__
        self.company_name = "Sistemas Agiles"
        self.copyright = module.__copyright__
        self.name = "Interfaz PyAfipWs - %s" % os.path.basename(
            module.__file__).replace(".pyc", ".py")



#!/usr/bin/python
# -*- coding: utf-8 -*-
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTIBILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.

"Módulo de Intefase para archivos de texto"

__author__ = "Mariano Reingart (mariano@nsis.com.ar)"
__copyright__ = "Copyright (C) 2009 Mariano Reingart"
__license__ = "GPL 3.0"
__version__ = "1.24e"

import os
import sys
import time
import traceback
from ConfigParser import SafeConfigParser

# revisar la instalación de pyafip.ws:
import wsaa,wsfe
from php import SimpleXMLElement, SoapClient, SoapFault, date


HOMO = False
DEBUG = False
CONFIG_FILE = "rece.ini"

LICENCIA = """
rece.py: Interfaz de texto para generar Facturas Electrónicas
Copyright (C) 2008/2009 Mariano Reingart reingart@gmail.com

Este progarma es software libre, se entrega ABSOLUTAMENTE SIN GARANTIA
y es bienvenido a redistribuirlo bajo la licencia GPLv3.

Para información adicional sobre garantía, soporte técnico comercial
e incorporación/distribución en programas propietarios ver PyAfipWs:
http://www.sistemasagiles.com.ar/trac/wiki/PyAfipWs
"""

# definición del formato del archivo de intercambio:
N = 'Numerico'
A = 'Alfanumerico'
I = 'Importe'
FORMATO = { 'tipo_doc': (36,2,N), 'nro_doc':  (38,11,N),
            'tipo_cbte': (10,2,N), 'punto_vta': (13,4,N),
            'cbt_desde': (17,8,N), 'cbt_hasta': (25,8,N),
            'imp_total': (79,15,I), 'imp_tot_conc': (94,15,I),
            'imp_neto': (109,15,I), 'impto_liq': (124,15,I),
            'impto_liq_rni': (139,15,I), 'imp_op_ex': (154,15,I),
            'fecha_cbte': (2,8,A),
            'cae': (261,14,N), 'fecha_vto': (275,8,A),
            'resultado': (291,1,A), 'motivo': (292,2,A), 'reproceso': (294,1,A),
            'fecha_venc_pago': (295,8,A),
            'presta_serv': (303,1,N),
            'fecha_serv_desde': (304,8,A),
            'fecha_serv_hasta': (312,8,A),
            'id': (320,15,N)
            }

def leer(linea, formato):
    dic = {}
    for clave, (comienzo, longitud, tipo) in formato.items():
        valor = linea[comienzo-1:comienzo-1+longitud].strip()
        try:
            if tipo == N and valor:
                valor = str(int(valor))
            if tipo == I and valor:
                valor = "%s.%02d" % (int(valor[:-2]), int(valor[-2:]))
            dic[clave] = valor
        except Exception as e:
            raise ValueError("Error al leer campo %s pos %s val '%s': %s" % (
                clave, comienzo, valor, str(e)))
    return dic

def escribir(dic, formato):
    linea = " " * 335
    for clave, (comienzo, longitud, tipo) in formato.items():
        valor = dic.get(clave,"")
        if tipo == N and valor and valor!="NULL":
            valor = ("%%0%dd" % longitud) % int(valor)
        elif tipo == I and valor:
            valor = ("%%0%dd" % longitud) % int(valor.replace(".",""))
        else:
            valor = ("%%0%ds" % longitud) % valor
        linea = linea[:comienzo-1] + valor + linea[comienzo-1+longitud:]
    return linea + "\n"

def autenticar(cert, privatekey, url):
    "Obtener el TA"
    TA = "TA-wsfe.xml"
    ttl = 60*60*5
    if not os.path.exists(TA) or os.path.getmtime(TA)+(ttl)<time.time():
        import wsaa
        tra = wsaa.create_tra(service="wsfe",ttl=ttl)
        cms = wsaa.sign_tra(str(tra),str(cert),str(privatekey))
        ta_string = wsaa.call_wsaa(cms,wsaa_url,trace=DEBUG)
        open(TA,"w").write(ta_string)
    ta_string=open(TA).read()
    ta = SimpleXMLElement(ta_string)
    token = str(ta.credentials.token)
    sign = str(ta.credentials.sign)
    return token, sign

def autorizar(client, token, sign, cuit, entrada, salida, formato):
    # recupero el último número de transacción
    ##id = wsfe.ultnro(client, token, sign, cuit)
    for linea in entrada:
        kargs = leer(linea, FORMATO)
        if not kargs['id'].strip():
            # TODO: habria que leer y/o grabar el id en el archivo
            ##id += 1 # incremento el nº de transacción
            # Por el momento, el id se calcula con el tipo, pv y nº de comprobant
            id = long(kargs['cbt_desde'])
            id += (int(kargs['tipo_cbte'])*10**4 + int(kargs['punto_vta']))*10**8
            kargs['id'] = id
        if DEBUG:
            print('\n'.join(["%s='%s'" % (k,v) for k,v in kargs.items()]))
        if not DEBUG or raw_input("Facturar?")=="S":
            ret = wsfe.aut(client, token, sign, cuit,
                **kargs)
            kargs.update(ret)
            salida.write(escribir(kargs, FORMATO))
            print("ID:", kargs['id'], "CAE:",kargs['cae'],"Motivo:",kargs['motivo'],"Reproceso:",kargs['reproceso'])


def depurar_xml(client):
    fecha = time.strftime("%Y%m%d%H%M%S")
    f=open("request-%s.xml" % fecha,"w")
    f.write(client.xml_request)
    f.close()
    f=open("response-%s.xml" % fecha,"w")
    f.write(client.xml_response)
    f.close()


if __name__ == "__main__":
    if '/ayuda' in sys.argv:
        print(LICENCIA)
        print
        print("Opciones: ")
        print(" /ayuda: este mensaje")
        print(" /dummy: consulta estado de servidores")
        print(" /prueba: genera y autoriza una factura de prueba (no usar en producción!)")
        print(" /ult: consulta último número de comprobante")
        print(" /debug: modo depuración (detalla y confirma las operaciones)")
        print("Ver rece.ini para parámetros de configuración (URL, certificados, etc.)")
        sys.exit(0)

    config = SafeConfigParser()
    config.read(CONFIG_FILE)
    cert = config.get('WSAA','CERT')
    privatekey = config.get('WSAA','PRIVATEKEY')
    cuit = config.get('WSFE','CUIT')
    entrada = config.get('WSFE','ENTRADA')
    salida = config.get('WSFE','SALIDA')

    if config.has_option('WSAA','URL') and not HOMO:
        wsaa_url = config.get('WSAA','URL')
    else:
        wsaa_url = wsaa.WSAAURL
    if config.has_option('WSFE','URL') and not HOMO:
        wsfe_url = config.get('WSFE','URL')
    else:
        wsfe_url = wsfe.WSFEURL

    if '/debug'in sys.argv:
        DEBUG = True

    XML = '/xml' in sys.argv

    if DEBUG:
        print("wsaa_url %s\nwsfe_url %s" % (wsaa_url, wsfe_url))

    try:
        client = SoapClient(wsfe_url, action=wsfe.SOAP_ACTION, namespace=wsfe.SOAP_NS,
                            trace=DEBUG, exceptions=True)

        if '/dummy' in sys.argv:
            print("Consultando estado de servidores...")
            wsfe.dummy(client)
            sys.exit(0)

        # TODO: esto habría que guardarlo en un archivo y no tener que autenticar cada vez
        token, sign = autenticar(cert, privatekey, wsaa_url)

        if '/prueba' in sys.argv or False:
            # generar el archivo de prueba para la próxima factura
            fecha = date('Ymd')
            tipo_cbte = 1
            punto_vta = 2
            ult_cbte = wsfe.recuperar_last_cmp(client, token, sign, cuit, punto_vta, tipo_cbte)
            f_entrada = open(entrada,"w")
            for cbte in range(ult_cbte+1,ult_cbte+4):
                dic = dict(
                    fecha=fecha,
                    presta_serv = 1,
                    tipo_doc = 80, nro_doc = "23111111114",
                    tipo_cbte=tipo_cbte, punto_vta=punto_vta,
                    cbt_desde=cbte, cbt_hasta=cbte,
                    imp_total="121.00", imp_tot_conc="0.00", imp_neto="100.00",
                    impto_liq="21.00", impto_liq_rni="0.00", imp_op_ex="0.00",
                    fecha_cbte=fecha, fecha_venc_pago=fecha,
                    fecha_serv_desde=fecha, fecha_serv_hasta=fecha)
                f_entrada.write(escribir(dic, FORMATO))
            f_entrada.close()

        if '/ult' in sys.argv:
            print("Consultar ultimo numero:")
            tipo_cbte = int(raw_input("Tipo de comprobante: "))
            punto_vta = int(raw_input("Punto de venta: "))
            ult_cbte = wsfe.recuperar_last_cmp(client, token, sign, cuit, punto_vta, tipo_cbte)
            print("Ultimo numero: ", ult_cbte)
            sys.exit(0)

        f_entrada = f_salida = None
        try:
            f_entrada = open(entrada,"r")
            f_salida = open(salida,"w")
            autorizar(client, token, sign, cuit, f_entrada, f_salida, FORMATO)
        finally:
            if f_entrada is not None: f_entrada.close()
            if f_salida is not None: f_salida.close()
            if XML:
                depurar_xml(client)

        sys.exit(0)

    except SoapFault as e:
        print(e.faultcode, e.faultstring.encode("ascii", "ignore"))
        sys.exit(3)
    except wsfe.WSFEError as e:
        print(e.code, e.msg.encode("ascii","ignore"))
        sys.exit(4)
    except Exception as e:
        print(e)
        sys.exit(5)



#!/usr/bin/python
# -*- coding: utf-8 -*-
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTIBILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.

"M�dulo para enviar correos electr�nicos"

__author__ = "Mariano Reingart <reingart@gmail.com>"
__copyright__ = "Copyright (C) 2011 Mariano Reingart"
__license__ = "GPL 3.0"
__version__ = "1.06f"

import os
import sys
import traceback

from email.mime.text import MIMEText
from email.mime.application import MIMEApplication
from email.mime.multipart import MIMEMultipart
import sys, os
import smtplib
from configparser import SafeConfigParser


DEBUG = False


class PyEmail:
    "Interfaz para enviar correos de Factura Electr�nica"
    _public_methods_ = ['Conectar', 'Crear', 'Enviar',
                        'AgregarDestinatario', 'Adjuntar',
                        'AgregarCC', 'AgregarBCC',
                        ]
    _public_attrs_ = [
                    'Motivo', 'Remitente', 'Destinatarios', 'ResponderA',
                    'MensajeHTML', 'MensajeTexto',
                    'Version', 'Excepcion', 'Traceback',
                    ]

    _reg_progid_ = "PyEmail"
    _reg_clsid_ = "{2BEF3037-BF38-41AA-84A3-6F109D543FC9}"


    def __init__(self):
        self.Version = __version__
        self.Excepcion = self.Traceback = ""
        self.Motivo = self.Destinatario = self.ResponderA = ""
        self.MensajeHTML = MensajeTexto = None
        self.adjuntos = []
        self.BCC = []
        self.CC = []

    def Conectar(self, servidor, usuario=None, clave=None, puerto=25):
        "Iniciar conexi�n al servidor de correo electronico"
        try:
            # convertir el nro de puerto a entero porque puede ser string:
            puerto = int(puerto)
            if puerto != 465:
                self.smtp = smtplib.SMTP(servidor, puerto)
            else:
                # creo una conexi�n segura (SSL, no disponible en Python<2.6):
                self.smtp = smtplib.SMTP_SSL(servidor, puerto)
            if DEBUG:
                self.smtp.set_debuglevel(1)
            self.smtp.ehlo()
            if puerto == 587:
                # inicio una sesi�n segura (TLS)
                self.smtp.starttls()
            if usuario and clave:
                # convertir a string (hmac necesita string "bytes")
                if isinstance(usuario, str):
                    usuario = usuario.encode("utf8")
                if isinstance(clave, str):
                    clave = clave.encode("utf8")
                self.smtp.login(usuario, clave)
            return True
        except Exception as e:
            ex = traceback.format_exception( sys.exc_info()[0], sys.exc_info()[1], sys.exc_info()[2])
            self.Traceback = ''.join(ex)
            self.Excepcion = traceback.format_exception_only( sys.exc_info()[0], sys.exc_info()[1])[0]
            return False

    def Crear(self, remitente="", motivo=""):
        "Inicializa un mensaje de correo"
        self.Remitente = remitente
        self.Motivo = motivo
        self.Destinatarios = []
        self.adjuntos = []
        return True

    def AgregarDestinatario(self, destinatario):
        "Agrega una direcci�n de correo de destino"
        self.Destinatarios.append(destinatario)
        return True

    def AgregarCC(self, destinatario):
        "Agrega una direcci�n de correo de destino (copia carb�nica)"
        self.CC.append(destinatario)
        return True

    def AgregarBCC(self, destinatario):
        "Agrega una direcci�n de correo de destino (copia carb�nica invisible)"
        self.BCC.append(destinatario)
        return True

    def Adjuntar(self, archivo):
        "Agrega un archivo para ser enviado como adjunto"
        self.adjuntos.append(archivo)
        return True

    def Enviar(self, remitente="", motivo="", destinatario="", mensaje="", archivo=None):
        "Generar un correo multiparte y enviarlo"
        try:
            to = ([destinatario] if destinatario
                  else self.Destinatarios)

            msg = MIMEMultipart('related')
            msg['Subject'] = motivo or self.Motivo
            msg['From'] = remitente or self.Remitente
            msg['Reply-to'] = remitente or self.ResponderA
            msg['To'] = ', '.join(to)
            if self.CC:
                msg['CC'] = ", ".join(self.CC)
                to += self.CC
            if self.BCC:
                to += self.BCC

            msg.preamble = 'Mensaje de multiples partes.\n'

            if mensaje:
                text = mensaje
                html = None
            else:
                text = self.MensajeTexto
                html = self.MensajeHTML

            if html:
                alt = MIMEMultipart('alternative')
                msg.attach(alt)
                part = MIMEText(text, 'text')
                alt.attach(part)
                part = MIMEText(html, 'html')
                alt.attach(part)
            else:
                part = MIMEText(text)
                msg.attach(part)

            if archivo:
                self.adjuntos.append(archivo)

            for archivo in self.adjuntos:
                part = MIMEApplication(open(archivo,"rb").read())
                part.add_header('Content-Disposition', 'attachment',
                                    filename=os.path.basename(archivo))
                msg.attach(part)

            # print("Enviando email: %s a %s" % (msg['Subject'], msg['To']))
            self.smtp.sendmail(msg['From'], to, msg.as_string())

            return True
        except Exception as e:
            ex = traceback.format_exception( sys.exc_info()[0], sys.exc_info()[1], sys.exc_info()[2])
            self.Traceback = ''.join(ex)
            self.Excepcion = traceback.format_exception_only( sys.exc_info()[0], sys.exc_info()[1])[0]
            return False

    def Salir(self):
        "Termino la conexi�n al servidor de correo electronico"
        try:
            self.smtp.quit()
            return True
        except Exception as e:
            ex = traceback.format_exception( sys.exc_info()[0], sys.exc_info()[1], sys.exc_info()[2])
            self.Traceback = ''.join(ex)
            self.Excepcion = traceback.format_exception_only( sys.exc_info()[0], sys.exc_info()[1])[0]
            return False


if __name__ == '__main__':

    if "--register" in sys.argv or "--unregister" in sys.argv:
        import win32com.server.register
        win32com.server.register.UseCommandLine(PyEmail)
    elif "py2exe" in sys.argv:
        from distutils.core import setup
        from .nsis import build_installer, Target
        import py2exe
        setup(
            name="PyEmail",
            version=__version__,
            description="Interfaz PyAfipWs Email %s",
            long_description=__doc__,
            author="Mariano Reingart",
            author_email="reingart@gmail.com",
            url="http://www.sistemasagiles.com.ar",
            license="GNU GPL v3",
            com_server = ['pyemail'],
            console=[],
            options={
                'py2exe': {
                'includes': ['email.generator', 'email.iterators', 'email.message', 'email.utils'],
                'optimize': 2,
                'excludes': ["pywin", "pywin.dialogs", "pywin.dialogs.list", "win32ui","distutils.core","py2exe","nsis"],
                #'skip_archive': True,
            }},
            data_files = [(".", ["licencia.txt"]),],
            cmdclass = {"py2exe": build_installer}
        )
    elif "/Automate" in sys.argv:
        # MS seems to like /automate to run the class factories.
        import win32com.server.localserver
        #win32com.server.localserver.main()
        # start the server.
        win32com.server.localserver.serve([PyEmail._reg_clsid_])
    elif "/prueba" in sys.argv:
        pyemail = PyEmail()
        import getpass
        usuario = input("usuario:")
        clave = getpass.getpass("clave:")
        ok = pyemail.Conectar("smtp.gmail.com", "reingart", clave, 587)
        print("login ok?", ok, pyemail.Excepcion)
        print(pyemail.Traceback)
        ok = pyemail.Enviar(usuario, "prueba", usuario, "prueba!", None)
        print("mail enviado?", ok, pyemail.Excepcion)
        ok = pyemail.Salir()
    else:
        config = SafeConfigParser()
        config.read("rece.ini")

        if '/debug'in sys.argv:
            DEBUG = True
            print("VERSION", __version__)
            sys.argv.remove("/debug")

        if len(sys.argv)<3:
            print("Par�metros: motivo destinatario [mensaje] [archivo]")
            sys.exit(1)

        conf_mail = dict(config.items('MAIL'))
        motivo = sys.argv[1]
        destinatario = sys.argv[2]
        mensaje = len(sys.argv)>3 and sys.argv[3] or conf_mail['cuerpo']
        archivo = len(sys.argv)>4 and sys.argv[4] or None

        print("Motivo: ", motivo)
        print("Destinatario: ", destinatario)
        print("Mensaje: ", mensaje)
        print("Archivo: ", archivo)

        pyemail = PyEmail()
        ok = pyemail.Conectar(conf_mail['servidor'],
                              conf_mail['usuario'], conf_mail['clave'],
                              conf_mail.get('puerto', 25))
        if ok:
            pyemail.Enviar(conf_mail['remitente'],
                           motivo, destinatario, mensaje, archivo)
        else:
            print(pyemail.Traceback)
